require 'json'
require 'tempfile'

class RspecPact
  class << self
    def generate_pact_file(description, request, response)
      write(pact(interactions(interaction(description, request, response))))
    end

    private

    def pact(interactions)
      {
        "consumer": { "name": '' },
        "provider": { "name": '' },
        "interactions": interactions,
        "metadata": {
          "pactSpecification": {
            "version": '2.0.0'
          }
        }
      }
    end

    def interaction(description, request, response)
      {
        "description": description,
        "request": parse_request(request),
        "response": parse_response(response)
      }
    end

    def parse_request(request)
      {
        "method":  request.method,
        "path":    request.path_info,
        "query":   URI(request.fullpath).query,
        "headers": { "Content-Type": request.headers['Content-Type'] },
        "body":    (JSON.parse(request.body.string) if request.body.string.present?)
      }
    end

    def parse_response(response)
      {
        "status": response.status,
        "headers": {
          "Content-Type": response.headers['Content-Type']
        },
        "body": (JSON.parse(response.body) if response.body.present?)
      }
    end

    def interactions(data)
      [data]
    end

    def write(data)
      pact_file = Tempfile.new('pact_file.json')
      File.open(pact_file, 'w+') do |f|
        f.write(JSON.generate(data))
      end
      pact_file.path
    end
  end
end

